#include "database.h"
#include <QSqlDatabase>
#include <QSqlError>
#include <QtConcurrent>

Database::Database(QObject *parent)
    : QObject{parent}
    , m_pool{}
{
    m_pool.setMaxThreadCount(1);
    m_pool.setExpiryTimeout(-1);
}

QFuture<void> Database::open(const QString &user, const QString &password)
{
    return QtConcurrent::run(&m_pool, [this, user, password]() {
        QString errorMessage;
        QString connectionName;
        {
            QSqlDatabase db = QSqlDatabase::addDatabase("QPSQL");
            db.setConnectOptions("service=camper; options=-csearch_path=camper,public");
            if (db.open(user, password)) {
                emit opened();
                return;
            }
            errorMessage = db.lastError().text();
            connectionName = db.connectionName();
        }
        QSqlDatabase::removeDatabase(connectionName);
        emit errorOcurred(errorMessage);
    });
}

QFuture<void> Database::close()
{
    return QtConcurrent::run(&m_pool, [this]() {
        QString connectionName;
        {
            QSqlDatabase db = QSqlDatabase::database();
            if (!db.isValid()) {
                return;
            }
            connectionName = db.connectionName();
            db.close();
        }
        QSqlDatabase::removeDatabase(connectionName);
        emit closed();
    });
}

#include "moc_database.cpp"
